<?php
/*
Plugin Name: WordPress Admin Restrictor
Plugin URI: http://www.towerwebdesign.co.uk/wordpress-admin-restrictor
Description: Restricts admin access to known ip addresses
Author: Richard Brown
Version: 1.0.0
Author URI: http://towerwebdesign.co.uk/
*/

// Do the magic stuff
add_action( 'plugins_loaded', 'wordpress_admin_restrictor_ban' );

add_action( 'admin_init', 'wordpress_admin_restrictor_ban_init' );
add_action('admin_menu', 'register_wordpress_admin_restrictor_ban_submenu_page');
   
function wordpress_admin_restrictor_ban_init() {
   wp_enqueue_style('admin-restrictor');
}

function register_wordpress_admin_restrictor_ban_submenu_page() {
    add_submenu_page( 
        'options-general.php', __('Admin Restrictor'), __('Admin Restrictor'), 
        'manage_options', 
        'wp-admin-restrictor', 
        'wordpress_admin_restrictor_ban_callback' ); 
}

function wordpress_admin_restrictor_ban_callback() {

    // By Default activate do not redirect for logged in users
    if (!get_option('wp_admin_not_for_logged_in_user'))    update_option('wp_admin_not_for_logged_in_user', 1);

    // form submit  and save values
    if (isset( $_POST['_wpprotect'] ) 
        && wp_verify_nonce( $_POST['_wpprotect'], 'ipbanlist' ) ) {
		  $allowed_html				= array();
        $ip_list                = wp_kses($_POST['ip_list'],$allowed_html);
        $redirect_url           = sanitize_text_field($_POST['redirect_url']);
        $not_for_logged_in_user = sanitize_text_field($_POST['not_for_logged_in_user']);

        update_option('wp_admin_ip_list',                $ip_list);
        update_option('wp_admin_redirect_url',           $redirect_url);
        update_option('wp_admin_not_for_logged_in_user', $not_for_logged_in_user);
    }

    // read values from option table

	$ip_list      = get_option('wp_admin_ip_list');
	$redirect_url = get_option('wp_admin_redirect_url');
	$not_for_logged_in_user = (intval(get_option('wp_admin_not_for_logged_in_user')) == 1 ) ? 1 : 0;
?>

<div class="wrap" style="overflow: hidden;"'>
    <div class="icon32" id="icon-options-general"><br></div><h2><?php _e('WordPress Admin Restrictor'); ?></h2>

    <p>
        <?php _e('Add ip addresses in the textarea. One IP address per line. 
        You may specify a redirect URL; when a user from a banned IP accesses your site, 
        they will be redirected to the specified URL.' ) ?>
    </p>    

    <form action="" method="post">

    <p>
    <label for='ip-list'><?php _e('IP List'); ?></label> <br/>
    <textarea name='ip_list' id='ip-list'><?php echo $ip_list ?></textarea>
    <p>

    <p>
    <label for='redirect-url'><?php _e('Redirect URL'); ?></label> <br/>
    <input  type='url' name='redirect_url' id='redirect-url' 
            value='<?php echo $redirect_url; ?>' 
            placeholder='<?php _e('Enter a valid URL') ?>' />
    <p>
    <p>
    <label for='not-for-logged-in-user'><?php _e('Do Not Redirect for Logged In User'); ?></label> <br/>
    <input  type='checkbox' name='not_for_logged_in_user' id='not-for-logged-in-user' 
            value='1' 
            <?php echo ($not_for_logged_in_user == 1 )  ? " checked='checked'" : "" ?>
             />
             <br/>
             <small><?php _e('If this box is checked the IP ban will be disabled for logged in users.') ?></small>
    <p>

    <?php wp_nonce_field('ipbanlist', '_wpprotect') ?>

    <p>
        <input type='submit' name='submit' value='<?php _e('Save') ?>' />
    </p>


    </form>

</div>

<?php

}

function admin_page_requested() {
	$url = $_SERVER['REQUEST_URI'];
	if (strpos(strtolower($url),'wp-log')!==false)
		return true;
	return strpos(strtolower($url),'wp-admin')!==false;
}

function war_SanitizeInputString($input_string) {
   return htmlentities(trim($input_string),ENT_NOQUOTES);
}

function findRequestIP() {
	if (isset($_SERVER['HTTP_CF_CONNECTING_IP']))		// cloudflare ip
		return war_SanitizeInputString($_SERVER['HTTP_CF_CONNECTING_IP']);
		
	if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {		// forwarded ip
		$ip = war_SanitizeInputString($_SERVER['HTTP_X_FORWARDED_FOR']);
		if ($ip!='' && $ip[0]==',')
			$ip = substr($ip,1);
		return trim($ip);
	}
	
	return war_SanitizeInputString($_SERVER['REMOTE_ADDR']);
}

function wordpress_admin_restrictor_ban() {
    // Do nothing for admin user
    if ((is_user_logged_in() && is_admin()) ||
        (intval(get_option('wp_admin_not_for_logged_in_user')) == 1  && is_user_logged_in())) return '';
        
    // we only want to restrict wp-admin pages
    if (!admin_page_requested())
		return '';
		
   $remote_ip = findRequestIP(); 

	if (wp_admin_check_ip_address($remote_ip, get_option('wp_admin_ip_list'))) {
		return '';
	} else {
		//print("wp_admin_check_ip_address fails<br />");
		$redirect_url = get_option('wp_admin_redirect_url');
		if (wordpress_admin_restrictor_ban_get_current_url() == $redirect_url )
			return '';
		wp_redirect( $redirect_url );			// admin access not allowed, so redirect away
      exit;
	}
}

/**
 * Check for a given ip address. 
 *
 * @param: string $ip The ip adddress
 * @param: string $ip_list The list with the banned ip addresss
 *
 * @return: boolean If founded it will return true, otherwise false
 **/

function wp_admin_check_ip_address($ip, $ip_list) {
    
    $list_arr = explode("\n", $ip_list);

    // Check for exact IP
    if (in_array($ip, $list_arr)) return true;
    if (strpos($ip_list,$ip)!==false)
			return true;

    // Check in IP range
    foreach ($list_arr as $k => $v) {
		 //print("Checking $k => $v<br />");
        if (substr_count($v, '-')) {
            // It's an ip range
            $curr_ip_range = explode('-', $v);
            if (ip2long($ip) >= ip2long(trim($curr_ip_range[0])) && 
                ip2long($ip) <= ip2long(trim($curr_ip_range[1] ))) return true;
        }   
    }

    return false;
}

function wordpress_admin_restrictor_ban_get_current_url() {
	$pageURL = (@$_SERVER["HTTPS"] == "on") ? "https://" : "http://";
	if ($_SERVER["SERVER_PORT"] != "80")
	{
	    $pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"];
	}
	else
	{
	    $pageURL .= $_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
	}
	return $pageURL;
}
